# -*- coding: utf-8 -*-
"""Init and utils."""
from zope.i18nmessageid import MessageFactory

import logging


_ = MessageFactory("collective.timestamp")
logger = logging.getLogger("collective.timestamp")
